#!/usr/bin/env python3
#
# Testowanie rozkladu S = s1*s1 + s2*s2, kiedy s1, s2 maja rozklad
# jednorodny w przedziale (-1,1).
# S ma rozklad jednorodny w przedziale (0,1).

n = 100
#n = 1000
D = dict()

for i in range(-n, n+1):
    for j in range(-n, n+1):
        if i*i+j*j < n*n:
            #D[i*i+j*j] = D.get(i*i+j*j, 0) +1
            D[(i*i+j*j)*10 / (n*n)] = D.get((i*i+j*j)*10 / (n*n), 0) +1
print(D)
for i in range(10):
    print(i, D.get(i,0))

# EOF



#!/usr/bin/env python3
#
# Based on Vesely (1982).
#
# Conclusions
# random_unit_quat() is 5 times faster that random_quat_biax(),
# but it is unclear if the distributions are similar.

import math
import random
import timeit
from pyquats.quats import Quat
from pyquats.qtools import random_quat_biax
from pyquats.qtools import random_unit_quat
from pyquats.qtools import random_move_quat
from pyquats.qtools import random_move_Xwall
from pyquats.qtools import random_move_Ywall
from pyquats.qtools import random_move_Zwall

for i in range(5):
    #q = random_unit_quat()
    #q = random_move_quat(0.5)
    q = random_move_Zwall(0.5)
    print(q)
    print(q.is_unit())
    print("abs {}".format(abs(q)))

N = 10000

print ( "random_quat_biax()..." )
t1 = timeit.Timer(lambda: random_quat_biax())
print ( "{} {}".format(N, t1.timeit(N)) )

print ( "random_unit_quat()..." )   # over 5 times faster
t1 = timeit.Timer(lambda: random_unit_quat())
print ( "{} {}".format(N, t1.timeit(N)) )

print ( "random_move_quat()..." )   # over 5 times faster
t1 = timeit.Timer(lambda: random_move_quat())
print ( "{} {}".format(N, t1.timeit(N)) )

print ( "random_move_Zwall()..." )   # over 5 times faster
t1 = timeit.Timer(lambda: random_move_Zwall(0.25))
print ( "{} {}".format(N, t1.timeit(N)) )

# EOF



#!/usr/bin/env python3

import sys
import unittest
import numpy as np
from pyquats.numpyquats import Quat


class TestQuat(unittest.TestCase):

    def setUp(self):
        self.zero = Quat()
        self.one = Quat(1)
        self.ii = Quat(0, 1, 0, 0)
        self.jj = Quat(0, 0, 1, 0)
        self.kk = Quat(0, 0, 0, 1)
        self.q1 = Quat(1, 2, 3, 4)
        self.q2 = Quat(1, 1, 1, 1)
        self.q3 = Quat(1.1, 2.2, 3.3, 4.4)
        self.q4 = Quat(-1.2, -2.3, -3.4, -4.5)
        self.c1 = 1J
        self.c2 = 1 + 2J

    def test_print(self):
        #self.assertEqual(str(self.zero),"0.0+0.0i+0.0j+0.0k")
        #self.assertEqual(str(self.q4),"-1.2-2.3i-3.4j-4.5k")
        self.assertEqual(repr(self.zero),"Quat(0.0, 0.0, 0.0, 0.0)")
        self.assertEqual(repr(self.q1),"Quat(1.0, 2.0, 3.0, 4.0)")
        self.assertEqual(repr(self.q4),"Quat(-1.2, -2.3, -3.4, -4.5)")

    def test_getitem(self):
        self.assertEqual(self.q1[0], 1)
        self.assertEqual(self.q1[1], 2)
        self.assertEqual(self.q1[2], 3)
        self.assertEqual(self.q1[3], 4)
        self.assertRaises(IndexError, self.q1.__getitem__, 5)

    def test_add(self):
        self.assertEqual(self.q1 + self.q2, Quat(2, 3, 4, 5))
        self.assertEqual(self.q1 + self.c1, Quat(1, 3, 3, 4))
        self.assertEqual(self.c2 + self.q2, Quat(2, 3, 1, 1))
        #print ( sys.getsizeof(self.q1) ) # 72 in Py2.7, 56 in Py3.7

    def test_sub(self):
        self.assertEqual(self.q1 - self.q2, Quat(0, 1, 2, 3))
        self.assertEqual(self.q1 - self.c2, Quat(0, 0, 3, 4))
        self.assertEqual(self.c2 - self.q2, Quat(0, 1, -1, -1))

    def test_mul(self):
        self.assertEqual(self.q1 * self.zero, self.zero)
        self.assertEqual(self.q1 * self.one, self.q1)
        # i*j=k, i*1=j*j=k*k=-1
        self.assertEqual(self.ii * self.jj, self.kk)
        self.assertEqual(self.jj * self.ii, Quat(0, 0, 0, -1))
        self.assertEqual(self.ii * self.ii, Quat(-1))
        self.assertEqual(self.jj * self.jj, Quat(-1))
        self.assertEqual(self.kk * self.kk, Quat(-1))
        # test complex
        self.assertEqual(self.c2 * Quat(3, 4), Quat(3, 4) * self.c2)
        self.assertEqual(self.q1 * self.c2, Quat(-3, 4, 11, -2))
        self.assertEqual(self.c2 * self.q1, Quat(-3, 4, -5, 10))

    def test_nonzero(self):
        self.assertTrue(bool(Quat(1)))
        self.assertFalse(bool(Quat()))
        self.assertEqual("a", "a" if Quat(1) else "b")
        self.assertEqual("b", "a" if Quat() else "b")

    def test_pos_neg(self):
        self.assertEqual(+self.q1, self.q1)
        self.assertEqual(-self.q1, Quat(-1, -2, -3, -4))
        self.assertEqual(self.jj * self.ii, -self.kk)
        self.assertEqual(self.ii * self.ii, -self.one)
        self.assertEqual(self.jj * self.jj, -self.one)
        self.assertEqual(self.kk * self.kk, -self.one)

    def test_is_unit(self):
        self.assertTrue(self.one.is_unit())
        self.assertTrue(self.ii.is_unit())
        self.assertTrue(self.jj.is_unit())
        self.assertTrue(self.kk.is_unit())
        self.assertTrue(Quat(0.5, 0.5, 0.5, 0.5).is_unit())
        self.assertFalse(self.q1.is_unit())
        self.assertFalse(self.q2.is_unit())

    def test_conjugate(self):
        self.assertEqual(self.q1.conjugate(), Quat(1, -2, -3, -4))
        self.assertEqual(self.one.conjugate(), self.one)

    def test_invert(self):
        self.assertEqual(~self.q2, Quat(0.25, -0.25, -0.25, -0.25))
        self.assertEqual(~self.one, self.one)

    def test_abs(self):
        self.assertEqual(abs(self.zero), 0)
        self.assertEqual(abs(self.one), 1)
        self.assertEqual(abs(self.ii), 1)
        self.assertEqual(abs(self.jj), 1)
        self.assertEqual(abs(self.kk), 1)
        self.assertEqual(abs(self.q2), 2)

    def test_pow(self):
        self.assertEqual(self.q1 ** 0, self.one)
        self.assertEqual(self.q1 ** 0, 1)
        self.assertEqual(self.q1 ** 1, self.q1)
        self.assertEqual(self.q1 ** 2, self.q1 * self.q1)
        self.assertEqual(self.ii ** 4, self.one)
        self.assertEqual(self.ii ** 400, self.one)

    def test_conversion(self):
        self.assertRaises(TypeError, int, self.one)
        #self.assertRaises(TypeError, long, self.one)   # Python 3
        self.assertRaises(TypeError, float, self.one)
        self.assertRaises(TypeError, complex, self.one)

    def test_hash(self):
        aset = set()
        aset.add(self.ii)
        aset.add(self.ii)  # ignored
        self.assertEqual(len(aset), 1)
        aset.add(self.jj)
        aset.add(self.kk)
        self.assertEqual(len(aset), 3)

    def test_from_rotations(self):
        self.assertAlmostEqual(Quat.from_x_rotation(np.pi), self.ii)
        self.assertAlmostEqual(Quat.from_y_rotation(np.pi), self.jj)
        self.assertAlmostEqual(Quat.from_z_rotation(np.pi), self.kk)

    def test_from_eulers(self):
        self.assertAlmostEqual(Quat.from_eulers(0, np.pi, 0), self.jj)
        self.assertAlmostEqual(Quat.from_eulers(np.pi, 0, 0), self.kk)
        self.assertAlmostEqual(Quat.from_eulers(0, 0, np.pi), self.kk)

    def test_random_quat(self):
        uniax = Quat.random_quat_uniax()
        biax = Quat.random_quat_biax()
        unit = Quat.random_unit_quat()
        move = Quat.random_move_quat(0.5)
        self.assertAlmostEqual(abs(uniax), 1)
        self.assertAlmostEqual(abs(biax), 1)
        self.assertAlmostEqual(abs(unit), 1)
        self.assertAlmostEqual(abs(move), 1)

    def tearDown(self): pass

if __name__== "__main__":

    unittest.main()

# EOF



#!/usr/bin/env python3

import unittest
import math
import cmath
import pyquats.qmath as qmath
from pyquats.quats import Quat


class TestQuatMath(unittest.TestCase):

    def setUp(self):
        self.f1 = 3.0
        self.f2 = 4.0
        self.c1 = 3J
        self.c2 = 1 + 2J

    def test_sgn(self):
        self.assertAlmostEqual(qmath.sgn(self.f1), Quat(1))
        self.assertAlmostEqual(qmath.sgn(-5), Quat(-1))
        self.assertAlmostEqual(qmath.sgn(self.c1), Quat(0, 1))
        self.assertAlmostEqual(abs(qmath.sgn(self.c2)), 1.0)
        self.assertAlmostEqual(qmath.sgn(Quat(3, 0, 4)), Quat(0.6, 0, 0.8))

    def test_exp(self):
        self.assertAlmostEqual(qmath.exp(self.f1), math.exp(self.f1))
        self.assertAlmostEqual(qmath.exp(self.c1), cmath.exp(self.c1))
        self.assertAlmostEqual(qmath.exp(self.c2), cmath.exp(self.c2))
        self.assertAlmostEqual(qmath.exp(Quat(self.f1)), math.exp(self.f1))
        self.assertAlmostEqual(qmath.exp(Quat(0, 3)), cmath.exp(3J))
        self.assertAlmostEqual(qmath.exp(Quat(4, 5)), cmath.exp(4 + 5J))
        self.assertAlmostEqual(qmath.exp(Quat(0, math.pi)), -1.0)
        self.assertAlmostEqual(qmath.exp(Quat(0, 0, math.pi)), -1.0)
        self.assertAlmostEqual(qmath.exp(Quat(0, 0, 0, math.pi)), -1.0)
        self.assertAlmostEqual(qmath.exp(Quat(self.f1, 0, self.f2)),
            math.exp(self.f1) * Quat(math.cos(self.f2), 0, math.sin(self.f2)))
        self.assertAlmostEqual(qmath.exp(Quat(self.f1, 0, 0, self.f2)),
            math.exp(self.f1) * Quat(math.cos(self.f2), 0, 0, math.sin(self.f2)))

    def test_log(self):
        self.assertAlmostEqual(qmath.log(self.f1), math.log(self.f1))
        self.assertAlmostEqual(qmath.log(self.c1), cmath.log(self.c1))
        self.assertAlmostEqual(qmath.log(self.c2), cmath.log(self.c2))

    def test_sin(self):
        self.assertAlmostEqual(qmath.sin(self.f1), math.sin(self.f1))
        self.assertAlmostEqual(qmath.sin(self.c1), cmath.sin(self.c1))
        self.assertAlmostEqual(qmath.sin(self.c2), cmath.sin(self.c2))
        self.assertAlmostEqual(qmath.sin(Quat(self.f1)), math.sin(self.f1))
        self.assertAlmostEqual(qmath.sin(Quat(0, 3)), cmath.sin(3J))
        self.assertAlmostEqual(qmath.sin(Quat(4, 5)), cmath.sin(4 + 5J))
        self.assertAlmostEqual(qmath.sin(Quat(0, self.f1)), Quat(0, math.sinh(self.f1)))
        self.assertAlmostEqual(qmath.sin(Quat(0, 0, self.f1)), Quat(0, 0, math.sinh(self.f1)))
        self.assertAlmostEqual(qmath.sin(Quat(0, 0, 0, self.f1)), Quat(0, 0, 0, math.sinh(self.f1)))

    def test_cos(self):
        self.assertAlmostEqual(qmath.cos(self.f1), math.cos(self.f1))
        self.assertAlmostEqual(qmath.cos(self.c1), cmath.cos(self.c1))
        self.assertAlmostEqual(qmath.cos(self.c2), cmath.cos(self.c2))
        self.assertAlmostEqual(qmath.cos(Quat(self.f1)), math.cos(self.f1))
        self.assertAlmostEqual(qmath.cos(Quat(0, 3)), cmath.cos(3J))
        self.assertAlmostEqual(qmath.cos(Quat(4, 5)), cmath.cos(4 + 5J))
        self.assertAlmostEqual(qmath.cos(Quat(0, self.f1)), Quat(math.cosh(self.f1)))
        self.assertAlmostEqual(qmath.cos(Quat(0, 0, self.f1)), Quat(math.cosh(self.f1)))
        self.assertAlmostEqual(qmath.cos(Quat(0, 0, 0, self.f1)), Quat(math.cosh(self.f1)))

    def test_sinh(self):
        self.assertAlmostEqual(qmath.sinh(self.f1), math.sinh(self.f1))
        self.assertAlmostEqual(qmath.sinh(self.c1), cmath.sinh(self.c1))
        self.assertAlmostEqual(qmath.sinh(self.c2), cmath.sinh(self.c2))
        self.assertAlmostEqual(qmath.sinh(Quat(self.f1)), math.sinh(self.f1))
        self.assertAlmostEqual(qmath.sinh(Quat(0, 3)), cmath.sinh(3J))
        self.assertAlmostEqual(qmath.sinh(Quat(4, 5)), cmath.sinh(4 + 5J))
        self.assertAlmostEqual(qmath.sinh(Quat(0, self.f1)), Quat(0, math.sin(self.f1)))

    def test_cosh(self):
        self.assertAlmostEqual(qmath.cosh(self.f1), math.cosh(self.f1))
        self.assertAlmostEqual(qmath.cosh(self.c1), cmath.cosh(self.c1))
        self.assertAlmostEqual(qmath.cosh(self.c2), cmath.cosh(self.c2))
        self.assertAlmostEqual(qmath.cosh(Quat(self.f1)), math.cosh(self.f1))
        self.assertAlmostEqual(qmath.cosh(Quat(0, 3)), cmath.cosh(3J))
        self.assertAlmostEqual(qmath.cosh(Quat(4, 5)), cmath.cosh(4 + 5J))
        self.assertAlmostEqual(qmath.cosh(Quat(0, self.f1)), Quat(math.cos(self.f1)))

    def tearDown(self): pass

if __name__== "__main__":

    unittest.main()

# EOF



#!/usr/bin/env python3

try:
    real_types = (int, long, float)
    range = xrange
except NameError:   # Python 3
    real_types = (int, float)

import math
import cmath
from pyquats.quats import Quat
#from pyquats.numpyquats import Quat

def sgn(x):
    """Return the sign of x, it is the unit quaternion for x != 0."""
    a = abs(x)
    if a == 0.0:
        return Quat(0)
    if isinstance(x, real_types):
        return Quat(x / a)
    elif isinstance(x, complex):
        return Quat(x.real / a, x.imag / a)
    else:
        return x * Quat(1.0 / a)


def exp(x):
    """Return the exponential value e^x."""
    if isinstance(x, real_types):
        return Quat(math.exp(x))
    elif isinstance(x, complex):
        # exp(a+bJ)=exp(a)*(cos(b)+sin(b)J)
        exp_real = math.exp(x.real)
        return Quat(exp_real * math.cos(x.imag), exp_real * math.sin(x.imag))
        #return Quat(1) * cmath.exp(x)
    else:   # isinstance(x, Quat)
        quat_imag = Quat(0, x.q[1], x.q[2], x.q[3])
        v = abs(quat_imag)
        #if v < 1e4:
        #    result = quat_imag * (1.0 - v*v/6.0)
        if v == 0.0:
            return Quat(math.exp(x.q[0]))
        else:
            result = quat_imag * Quat(math.sin(v) / v)
            result = result + Quat(math.cos(v))
            result = result * math.exp(x.q[0])
            return result


def log(x):
    """Return the natural logarithm (base e) of x."""
    if isinstance(x, real_types):
        return Quat(math.log(x))
    elif isinstance(x, complex):
        return Quat(1) * cmath.log(x)
    else:   # isinstance(x, Quat)
        raise NotImplementedError


def sin(x):
    """Return the sine of x.
    Note that Euler's formula is used to define sin(x).
    """
    if isinstance(x, real_types):
        return Quat(math.sin(x))
    elif isinstance(x, complex):
        #return Quat(1) * cmath.sin(x)
        return Quat(math.sin(x.real) * math.cosh(x.imag), 
                    math.cos(x.real) * math.sinh(x.imag))
    else:   # isinstance(x, Quat)
        quat_imag = Quat(0, x.q[1], x.q[2], x.q[3])
        v = abs(quat_imag)
        if v == 0.0:
            return Quat(math.sin(x.q[0]))
        else:
            quat_imag = quat_imag * Quat(1.0 / v)   # we get a unit quat
            quat = x * quat_imag
            result = exp(quat) - exp(-quat)
            result = result * quat_imag * Quat(-0.5)
            return result


def cos(x):
    """Return the cosine of x.
    Note that Euler's formula is used to define cos(x).
    """
    if isinstance(x, real_types):
        return Quat(math.cos(x))
    elif isinstance(x, complex):
        #return Quat(1) * cmath.cos(x)
        return Quat(math.cos(x.real) * math.cosh(x.imag), 
                   -math.sin(x.real) * math.sinh(x.imag))
    else:   # isinstance(x, Quat)
        quat_imag = Quat(0, x.q[1], x.q[2], x.q[3])
        v = abs(quat_imag)
        if v == 0.0:
            return Quat(math.cos(x.q[0]))
        else:
            quat_imag = quat_imag * Quat(1.0 / v)   # we get a unit quat
            quat = x * quat_imag
            result = exp(quat) + exp(-quat)
            result = result * Quat(0.5)
            return result


def sinh(x):
    """Return the hyperbolic sine of x."""
    if isinstance(x, real_types):
        return Quat(math.sinh(x))
    elif isinstance(x, complex):
        #return Quat(1) * cmath.sinh(x)
        return Quat(math.sinh(x.real) * math.cos(x.imag), 
                    math.cosh(x.real) * math.sin(x.imag))
    else:   # isinstance(x, Quat)
        # sinh(q)=(exp(q)-exp(-q))/2
        result = exp(x) - exp(-x)
        result = result * Quat(0.5)
        return result


def cosh(x):
    """Return the hyperbolic cosine of x."""
    if isinstance(x, real_types):
        return Quat(math.cosh(x))
    elif isinstance(x, complex):
        #return Quat(1) * cmath.cosh(x)
        return Quat(math.cosh(x.real) * math.cos(x.imag), 
                    math.sinh(x.real) * math.sin(x.imag))
    else:   # isinstance(x, Quat)
        # cosh(q)=(exp(q)+exp(-q))/2
        result = exp(x) + exp(-x)
        result = result * Quat(0.5)
        return result

# EOF



#!/usr/bin/env python3
#
# Results:
# math.sin(x) is 6x faster than np.sin(x).
# np.sum(np.sin(L)) is 2x faster than sum(math.sin() with for loop).

import random
import timeit
import math
import numpy as np

N = 1000000

L = [random.random()*10 for i in range(N)]

print ( "Testing sum(math.sin() with for loop)..." )
t1 = timeit.Timer(lambda: sum(math.sin(x) for x in L))
print ( "{} {}".format(N, t1.timeit(1)) )   # single run

print ( "Testing sum(np.sin() with for loop)..." )
t1 = timeit.Timer(lambda: sum(np.sin(x) for x in L))
print ( "{} {}".format(N, t1.timeit(1)) )   # single run

print ( "Testing sum(np.sin(L))..." )
t1 = timeit.Timer(lambda: sum(np.sin(L)))
print ( "{} {}".format(N, t1.timeit(1)) )   # single run

print ( "Testing np.sum(np.sin(L))..." )
t1 = timeit.Timer(lambda: np.sum(np.sin(L))) # BEST!
print ( "{} {}".format(N, t1.timeit(1)) )   # single run

# EOF



#!/usr/bin/env python3

import random
import functools
import timeit
from pyquats.quats import Quat
#from pyquats.numpyquats import Quat

N = 1000000

a = [Quat(i) for i in range(1, N+1)]

b = [Quat(random.uniform(-1,1), random.uniform(-1,1), random.uniform(-1,1),
    random.uniform(-1,1)).normalized() for i in range(N)]

print(sum(b))

print ( "Testing Quat (sum)..." )
t1 = timeit.Timer(lambda: sum(b))
print ( "{} {}".format(N, t1.timeit(1)) )   # single run

print ( "Testing Quat (sum+reduce)..." )
t1 = timeit.Timer(lambda: functools.reduce(lambda x,y: x+y, b, Quat(0)))
print ( "{} {}".format(N, t1.timeit(1)) )   # single run

print ( "Testing Quat (invert)..." )
t1 = timeit.Timer(lambda: [~q for q in b])
print ( "{} {}".format(N, t1.timeit(1)) )   # single run

print ( "Testing Quat (conjugate)..." )
t1 = timeit.Timer(lambda: [q.conjugate() for q in b])
print ( "{} {}".format(N, t1.timeit(1)) )   # single run

print ( "Testing Quat (neg)..." )
t1 = timeit.Timer(lambda: [-q for q in b])
print ( "{} {}".format(N, t1.timeit(1)) )   # single run

print(functools.reduce(lambda x,y: x-y, b, Quat(0)))

print ( "Testing Quat (neg+reduce)..." )
t1 = timeit.Timer(lambda: functools.reduce(lambda x,y: x-y, b, Quat(0)))
print ( "{} {}".format(N, t1.timeit(1)) )   # single run

print(functools.reduce(lambda x,y: x*y, b, Quat(1)))

print ( "Testing Quat (mul+reduce)..." )
t1 = timeit.Timer(lambda: functools.reduce(lambda x,y: x*y, b, Quat(1)))
print ( "{} {}".format(N, t1.timeit(1)) )   # single run

# EOF



#!/usr/bin/env python3

import math
import numpy
from pyquats.quats import Quat
from pyquats.qtools import random_quat_Xwall
from pyquats.qtools import random_quat_Ywall
from pyquats.qtools import random_quat_Zwall

#unit_quat = Quat(1) # R_z(0)
#unit_quat = Quat.rot_quat([0, 0, 1], 0) # R_z(0)
#unit_quat = Quat.rot_quat([0, 0, 1], math.pi) # R_z(pi)
#unit_quat = Quat.rot_quat([0, 1, 0], math.pi) # R_y(pi)
#unit_quat = Quat.rot_quat([1, 0, 0], math.pi) # R_x(pi)
#unit_quat = Quat.rot_quat([0, 0, 1], math.pi * 0.5) # R_z(pi/2)
#unit_quat = Quat.rot_quat([0, 1, 0], math.pi * 0.5) # R_y(pi/2)
#unit_quat = Quat.rot_quat([1, 0, 0], math.pi * 0.5) # R_x(pi/2)
#unit_quat = Quat(0.5, 0.5, 0.5, 0.5)   # os [1,1,1], kat 120 stopni
#unit_quat = random_quat_Xwall()   # vector l || X
#unit_quat = random_quat_Ywall()   # vector l || Y
unit_quat = random_quat_Zwall()   # vector l || Z

q0, q1, q2, q3 = unit_quat.q

l_x = 2.0 * (q0 * q0 + q1 * q1) - 1.0
l_y = 2.0 * (q0 * q3 + q1 * q2)
l_z = 2.0 * (q1 * q3 - q0 * q2)
m_x = 2.0 * (q1 * q2 - q0 * q3)
m_y = 2.0 * (q0 * q0 + q2 * q2) - 1.0
m_z = 2.0 * (q0 * q1 + q2 * q3)
n_x = 2.0 * (q0 * q2 + q1 * q3)
n_y = 2.0 * (q2 * q3 - q0 * q1)
n_z = 2.0 * (q0 * q0 + q3 * q3) - 1.0

# Step to Legendre P2().
p2l_x = 1.5 * l_x * l_x - 0.5
p2l_y = 1.5 * l_y * l_y - 0.5
p2l_z = 1.5 * l_z * l_z - 0.5
p2m_x = 1.5 * m_x * m_x - 0.5
p2m_y = 1.5 * m_y * m_y - 0.5
p2m_z = 1.5 * m_z * m_z - 0.5
p2n_x = 1.5 * n_x * n_x - 0.5
p2n_y = 1.5 * n_y * n_y - 0.5
p2n_z = 1.5 * n_z * n_z - 0.5

# Special functions oriented along Z.
F200 = p2n_z
F202 = (p2l_z - p2m_z) / math.sqrt(3)
F220 = (p2n_x - p2n_y) / math.sqrt(3)
F222 = (p2l_x + p2m_y - p2m_x - p2l_y) / 3.0

print ( "unit_quat", repr(unit_quat) )
print ( "vector l", l_x, l_y, l_z )
print ( "vector m", m_x, m_y, m_z )
print ( "vector n", n_x, n_y, n_z )
print ( "F200", F200 )
print ( "F202", F202 )
print ( "F220", F220 )
print ( "F222", F222 )

# Wybieranie spinow w kolejnych iteracjach.
# Mozna zrobic kilka obiegow po wszystkich spinach.
x, y, z = 3, 4, 5
for it in range(2*x*y*z):
    i = (it // (z*y)) % x
    j = (it % (z*y)) // z
    k = (it % (z*y)) % z
    #print (i, j, k)

# EOF
